<#
Author:  Oliver Kieselbach (oliverkieselbach.com)
Script:  Install-LanguageExperiencePack-de-DE.ps1
Description:
run in SYSTEM context, usage of MDM Bridge WMI Provider to install german language experience pack
Release notes:
Version 1.0: 2020-04-21 � Original published version.
The script is provided "AS IS" with no warranties.
Modify: Dataport TC23/17 01.03.2022 API Call + Params
Modify: Dataport TC23/17 28.03.2022 Status + Exit Codes
#>
[CmdletBinding()]
param (
    [Parameter(Mandatory=$true)][String]$AppID  #AppID from the Micrososft Store URL. Looks like this: 9n0dx20hk701
)

#Get PackageFamilyName from MS API
$DataURL = "https://bspmts.mp.microsoft.com/v1/public/catalog/Retail/Products/$($AppID)/applockerdata"
try {
    $packageFamilyName = (Invoke-RestMethod -Method Get -Uri $DataURL).packageFamilyName
}
catch [Exception] {
    write-error $_
    exit 1
}

#Definitions
$namespaceName = "root\cimv2\mdm\dmmap"
$session = New-CimSession
$skuId = 0016 #Standard Licence ID
$omaUri = "./Vendor/MSFT/EnterpriseModernAppManagement/AppInstallation"
$ClassName = "MDM_EnterpriseModernAppManagement_AppInstallation01_01"

#CIM Instance Settings
$newInstance = New-Object Microsoft.Management.Infrastructure.CimInstance $ClassName, $namespaceName
$property = [Microsoft.Management.Infrastructure.CimProperty]::Create("ParentID", $omaUri, "string", "Key")
$newInstance.CimInstanceProperties.Add($property)
$property = [Microsoft.Management.Infrastructure.CimProperty]::Create("InstanceID", $packageFamilyName, "String", "Key")
$newInstance.CimInstanceProperties.Add($property)

#parameter
$flags = 0 #Standard Flag
$paramValue = [Security.SecurityElement]::Escape($('<Application id="{0}" flags="{1}" skuid="{2}"/>' -f $AppID, $flags, $skuId))
$params = New-Object Microsoft.Management.Infrastructure.CimMethodParametersCollection
$param = [Microsoft.Management.Infrastructure.CimMethodParameter]::Create("param", $paramValue, "String", "In")
$params.Add($param)

try {
    # we create the MDM instance and trigger the StoreInstallMethod
    $instance = $session.CreateInstance($namespaceName, $newInstance)
    $result = $session.InvokeMethod($namespaceName, $instance, "StoreInstallMethod", $params)
    
    # Track Install Status for max 5 minutes (1 = Installing, 2 = Error, 3 = Installed, Removed Instance means Status 3)
    $i = 0
    do{
        Start-Sleep 1
        $Status = (Get-WmiObject -Namespace "root\cimv2\mdm\dmmap" -Class "MDM_EnterpriseModernAppManagement_AppInstallation01_01" | Where-Object InstanceID -eq $instance.InstanceID).Status
        $i++
    }until($i -ge 300 -or $Status -eq 2 -or $null -eq $Status)
}
catch [Exception] {
    write-error $_
    $Status = 2
}

Remove-CimSession -CimSession $session
if ($Status -eq 2) {
    exit 1
}else {
    exit 0
}
